# -*- coding: utf-8 -*-
"""
@author: Christoph Lassner
"""
import os
import cv2
import scipy.io as sio
import numpy as np
import cPickle as pickle

def read_chars74k_folder(foldername, filename, onlygood=False):
  matfile = sio.loadmat(os.path.join(foldername, filename))
  mlist = matfile['list']
  images = []
  annotations = []
  for im_idx, im_filename in enumerate(mlist['ALLnames'][0,0]):
    if onlygood:
      if not mlist['is_good'][0,0][im_idx,0] == 1:
        continue
    images.append(cv2.imread(os.path.join(foldername, mlist['ALLnames'][0,0][im_idx])+'.png', cv2.IMREAD_GRAYSCALE))
    assert not images[-1] is None
    annotations.append(mlist['ALLlabels'][0,0][im_idx,0])
  assert len(images) == len(annotations)
  return images, annotations

def load_chars74k_english(foldername):
  zipped_lists =  zip(read_chars74k_folder(os.path.join(foldername, 'Fnt'), 'lists_var_size.mat'),
                      read_chars74k_folder(os.path.join(foldername, 'Hnd'), 'lists_var_size.mat'),
                      read_chars74k_folder(os.path.join(foldername, 'Img'), 'lists.mat', onlygood=True))
  return map(lambda plist: reduce(lambda x, y: x+y, plist), zipped_lists)

def chars74k_english(foldername):
  cfilename = os.path.join(os.path.dirname(__file__), 'chars74k_cache.pickle')
  try:
    # Load from cache.
    with open(cfilename, 'r') as cfile:
      im_array, an_array = pickle.load(cfile)
  except:
    print 'Loading from cache failed. Constructing...'
    i, a = load_chars74k_english(foldername)
    im_array = np.concatenate(map(lambda im: np.reshape(cv2.resize(im, (8,8)), (1, 64)), i))
    an_array = np.array(a, dtype='uint')
    try:
      with open(cfilename, 'w') as cfile:
        pickle.dump((im_array, an_array), cfile)
        print 'Dumped to cachefile.'
    except:
      print 'Caching failed.'
  return im_array, an_array

if __name__ == "__main__":
  i, a = chars74k_english(os.path.join(os.path.dirname(__file__),
                                       'chars74k',
                                       'English'))
