# -*- coding: utf-8 -*-
#pylint: disable=W0311
"""
@author: Christoph Lassner
"""

from __future__ import print_function
import os
import fertilized_sklearn_helpers
from fertilized_sklearn_helpers import fertilized
from fertilized_sklearn_helpers import ForestWrapper, RegForestWrapper
from sklearn.cross_validation import ShuffleSplit
from sklearn.cross_validation import cross_val_score
import sklearn.utils
import sklearn.metrics
import numpy as np

def evaluate_crossval(soil,
                      data,
                      annotations,
                      val_iter,
                      val_size,
                      max_depth,
                      n_features_per_node,
                      n_thresholds_per_node,
                      n_trees,
                      min_samples_per_split,
                      min_samples_per_leaf,
                      ent_names,
                      pvals,
                      dset_name,
                      allow_redraw=False,
                      test_data = None,
                      test_annot= None,
                      resfpref=''):
  if soil._res_type == fertilized.Result_Types.regression:
    res_suffix = '_reg' + resfpref
    fertilized_sklearn_helpers._REG_SOIL_ = soil
  else:
    res_suffix = '' + resfpref
    fertilized_sklearn_helpers._SOIL_ = soil
    n_classes = len(np.unique(annotations))
  rand_init = 1
  n_features = data.shape[1]
  res_fname = os.path.join(os.path.dirname(__file__),
                           'results%s.csv' % (res_suffix))
  if not os.path.isfile(res_fname):
    with open(res_fname, 'w') as resultfile:
      resultfile.write('dataset_name, q, ent_name, run, performance'+os.linesep)
  shannon_scores = None
  for ent_name in ent_names:
    for q in pvals:
      if soil._res_type == fertilized.Result_Types.regression:
        origin = RegForestWrapper( n_features,
                                   max_depth,  # max depth
                                   n_features_per_node, # n features per node
                                   n_thresholds_per_node, #n random thresholds per node
                                   n_trees, # n trees
                                   min_samples_per_leaf,   # min samples per leaf
                                   min_samples_per_split,   # min samples per split
                                   0.,  # min gain threshold
                                   allow_redraw, # allow redraw,
                                   1,   # random seed
                                   ent_name, # entropy type
                                   q    # induced entropy param
                                   )
        score_func_name = 'mean_squared_error'
      else:
        origin = ForestWrapper(n_classes,
                               n_features,
                               max_depth,  # max depth
                               n_features_per_node, # n features per node
                               n_thresholds_per_node, #n random thresholds per node
                               n_trees, # n trees
                               min_samples_per_leaf,   # min samples per leaf
                               min_samples_per_split,   # min samples per split
                               0.,  # min gain threshold
                               allow_redraw, # allow redraw,
                               1,   # random seed
                               ent_name, # entropy type
                               q    # induced entropy param
                               )
        score_func_name = 'f1'
      if test_data is None:
        cross_val = ShuffleSplit(data.shape[0],
                                 n_iter=val_iter,
                                 test_size=val_size,
                                 random_state=rand_init)
      if ent_name == 'shannon' and not shannon_scores is None:
        scores = shannon_scores
      else:
        if not test_data is None:
          scores = []
          for seed in xrange(1, val_iter+1):
            model = origin.set_params(random_seed=seed)
            model.fit(data, annotations)
            if score_func_name == 'f1':
              scores.append(sklearn.metrics.f1_score(test_annot, model.predict(test_data)))
            else:
              scores.append(sklearn.metrics.mean_squared_error(test_annot, model.predict(test_data)))
        else:
          scores = cross_val_score(origin,
                                   np.ascontiguousarray(data),
                                   np.ascontiguousarray(annotations),
                                   scoring=score_func_name,
                                   cv=cross_val,
                                   verbose=0)
        if ent_name == 'shannon':
          shannon_scores = scores
      for run, score in enumerate(scores):
        with open(res_fname, 'a') as resultfile:
          resultfile.write('%s, %f, %s, %d, %f' % (dset_name,
                                                   q,
                                                   ent_name,
                                                   run,
                                                   score) +\
                           os.linesep)
      print('.', end='')
    print(ent_name)
  return