# -*- coding: utf-8 -*-
"""
@author: Christoph Lassner
"""
import os
os.chdir(os.path.dirname(__file__))
import csv
import numpy as np
np.random.seed(1)
import sklearn.grid_search as grid_search
from sklearn.cross_validation import ShuffleSplit
import fertilized_sklearn_helpers as helpers
from evaluation_helper import evaluate_crossval
#%% Load data.
with open('uci-20070111-abalone.csv', 'r') as dataf:
  reader = csv.reader(dataf)
  rowvals = []
  for row in reader:
    gcode = 0
    if row[0] == 'I':
      gcode = 1
    elif row[0] == 'F':
      gcode = 2
    rowvals.append([gcode] + [float(x) for x in row[1:]])
DTAARR = np.array(rowvals, dtype='float32')
train_data = DTAARR[:3133, :-1]
train_annot = DTAARR[:3133, -1]
test_data = DTAARR[3133:, :-1]
test_annot = DTAARR[3133:, -1]
n_features = train_data.shape[1]
helpers._N_THREADS = 15
soil = helpers._REG_SOIL_
#%% Grid search.
param_grid = {'max_depth':[15, 20, 25], # 15
              'features_per_node':[7, 10, 15], # 4
              'thresholds_per_node':[4, 7, 10]} # 3
rand_init = 1
origin = helpers.RegForestWrapper(
                       n_features,
                       25,  # max depth
                       int(np.sqrt(n_features)), # n features per node
                       10,  # n random thresholds per node
                       100, # n trees
                       4,   # min samples per leaf
                       8,   # min samples per split
                       0.,  # min gain threshold
                       True, # allow redraw,
                       rand_init,   # random seed
                       'shannon', # entropy type
                       2.    # induced entropy param
                       )
cv = ShuffleSplit(train_data.shape[0],
                  n_iter=3,
                  test_size=0.25,
                  random_state=rand_init)
clf = grid_search.GridSearchCV(origin,
                               param_grid,
                               'mean_squared_error',
                               cv=cv,
                               verbose=10,
                               refit=True)
clf.fit(train_data, train_annot)
print 'Best parameters determined: ', clf.best_params_
best_params = clf.best_params_
# Our result:
#best_params = {'thresholds_per_node': 7,
#               'features_per_node': 7,
#               'max_depth': 15}
#%% Evaluate.
evaluate_crossval(soil,
                  train_data,
                  train_annot,
                  10, # iter
                  1044, #val size
                  best_params['max_depth'], # depth
                  best_params['features_per_node'], # features
                  best_params['thresholds_per_node'], # thresholds
                  100, # trees
                  8, # min samples split
                  4, # min samples leaf
                  ['shannon', 'renyi', 'tsallis', 'induced_p'],
                  list(np.linspace(0.1, 0.9, 9)) + \
                  [1.1, 1.5, 2., 2.5, 3., 3.5, 4., 4.5, 5., 5.5, 6., 7.],
                  'abalone',
                  allow_redraw=True,
                  test_data=test_data,
                  test_annot=test_annot,
                  resfpref='_val_test')
