# -*- coding: utf-8 -*-
"""
@author: Christoph Lassner
"""
import os
os.chdir(os.path.dirname(__file__))
import csv
import numpy as np
np.random.seed(1)
import sklearn.grid_search as grid_search
from sklearn.cross_validation import ShuffleSplit, train_test_split
import fertilized_sklearn_helpers as helpers
from fertilized_sklearn_helpers import fertilized
from evaluation_helper import evaluate_crossval
from collections import namedtuple
#%% Load data
import chars74k_helper
tmpstruct = namedtuple("tmpstruct", "data target")
dset = tmpstruct(*chars74k_helper.chars74k_english(os.path.join(os.path.dirname(__file__),
                                                                'chars74k',
                                                                'English')))
dset.target[:] = dset.target - 1
train_data, test_data, train_annot, test_annot = train_test_split(dset.data.astype('uint8'),
                                                                  dset.target.astype('uint'),
                                                                  test_size=7400)
n_features = train_data.shape[1]
n_classes = np.unique(train_annot).shape[0]
train_data = train_data.copy()
test_data = test_data.copy()
train_annot = train_annot.copy()
test_annot = test_annot.copy()
assert train_annot.max() < n_classes
assert np.all(np.unique(train_annot) == range(62))
assert np.all(np.unique(test_annot) == range(62))
soil = fertilized.Soil('uint8', 'uint8', 'uint',
                       fertilized.Result_Types.probabilities)
helpers._SOIL_ = soil
helpers._N_THREADS = 15
#%% Grid search.
#param_grid = {'max_depth':[15, 20, 25],
#              'features_per_node':[7, 10, 15],
#              'thresholds_per_node':[4, 7, 10]}
#rand_init = 1
#origin = helpers.ForestWrapper(
#                       n_classes,
#                       n_features,
#                       25,  # max depth
#                       int(np.sqrt(n_features)), # n features per node
#                       10,  # n random thresholds per node
#                       100, # n trees
#                       2,   # min samples per leaf
#                       4,   # min samples per split
#                       0.,  # min gain threshold
#                       True, # allow redraw,
#                       rand_init,   # random seed
#                       'shannon', # entropy type
#                       2.    # induced entropy param
#                       )
#cv = ShuffleSplit(train_data.shape[0],
#                  n_iter=3,
#                  test_size=0.25,
#                  random_state=rand_init)
#clf = grid_search.GridSearchCV(origin,
#                               param_grid,
#                               'f1',
#                               cv=cv,
#                               verbose=10,
#                               refit=True)
#clf.fit(train_data, train_annot)
#print 'Best parameters determined: ', clf.best_params_
#best_params = clf.best_params_
# Our result:
best_params = {'thresholds_per_node': 10,
               'features_per_node': 7,
               'max_depth': 20}
evaluate_crossval(soil,
                  train_data,
                  train_annot,
                  10, # val iter
                  7400, # val size
                  best_params['max_depth'], # max depth
                  best_params['features_per_node'], # features per node
                  best_params['thresholds_per_node'], # thresholds per node
                  100, # n trees
                  4, # n samples per split
                  2, # n samples per leaf
                  ['shannon', 'tsallis', 'renyi', 'induced_p'],
                  list(np.linspace(0.1, 0.9, 9)) + \
                  [1., 1.5, 2., 2.5, 3., 3.5, 4., 4.5, 5., 5.5, 6., 7.],
                  'chars74k',
                  allow_redraw=True,
                  test_data = test_data,
                  test_annot = test_annot,
                  resfpref = '_val_test')
