# -*- coding: utf-8 -*-
"""
@author: Christoph Lassner
"""
import os
os.chdir(os.path.dirname(__file__))
import numpy as np
np.random.seed(1)
import sklearn.grid_search as grid_search
from sklearn.cross_validation import ShuffleSplit, train_test_split
import fertilized_sklearn_helpers as helpers
from fertilized_sklearn_helpers import fertilized
from evaluation_helper import evaluate_crossval
from collections import namedtuple
import scipy.io as sio
#%% Load data
dset_mat = sio.loadmat('g50c.mat')
tmpstruct = namedtuple("tmpstruct", "data target")
dset = tmpstruct(dset_mat['X'], dset_mat['y'])
# {-1, 1} -> {0, 1}
dset.target[:] = dset.target.astype('float') * 0.5 + 0.5
train_data, test_data, train_annot, test_annot = train_test_split(dset.data.astype('float32'),
                                                                  dset.target.astype('uint'),
                                                                  test_size=500)
n_features = train_data.shape[1]
n_classes = np.unique(train_annot).shape[0]
train_data = train_data.copy()
test_data = test_data.copy()
train_annot = train_annot.copy()
test_annot = test_annot.copy()
assert train_annot.max() < n_classes
assert np.all(np.unique(train_annot) == range(2))
assert np.all(np.unique(test_annot) == range(2))
soil = fertilized.Soil()
helpers._SOIL_ = soil
helpers._N_THREADS = 15
#%% Grid search.
#param_grid = {'max_depth':[10, 15, 20],
#              'features_per_node':[7, 10, 15],
#              'thresholds_per_node':[4, 7, 10]}
#rand_init = 1
#origin = helpers.ForestWrapper(
#                       n_classes,
#                       n_features,
#                       25,  # max depth
#                       int(np.sqrt(n_features)), # n features per node
#                       10,  # n random thresholds per node
#                       100, # n trees
#                       2,   # min samples per leaf
#                       4,   # min samples per split
#                       0.,  # min gain threshold
#                       True, # allow redraw,
#                       rand_init,   # random seed
#                       'shannon', # entropy type
#                       2.    # induced entropy param
#                       )
#cv = ShuffleSplit(train_data.shape[0],
#                  n_iter=100,
#                  test_size=0.2,
#                  random_state=rand_init)
#clf = grid_search.GridSearchCV(origin,
#                               param_grid,
#                               'f1',
#                               cv=cv,
#                               verbose=10,
#                               refit=True)
#clf.fit(train_data, train_annot)
#print 'Best parameters determined: ', clf.best_params_
#best_params = clf.best_params_
# Best parameters determined:  {'thresholds_per_node': 4, 'features_per_node': 4, 'max_depth': 15}
# best score: 0.87225568875568893
# Best parameters determined:  {'thresholds_per_node': 2, 'features_per_node': 2, 'max_depth': 10}
# 0.87625530025530041
# Our result:
best_params = {'thresholds_per_node': 4,
               'features_per_node': 7,
               'max_depth': 10}
evaluate_crossval(soil,
                  train_data,
                  train_annot,
                  250, # val iter
                  500, # val size
                  best_params['max_depth'], # max depth
                  best_params['features_per_node'], # features per node
                  best_params['thresholds_per_node'], # thresholds per node
                  100, # n trees
                  4, # n samples per split
                  2, # n samples per leaf
                  ['shannon', 'tsallis', 'renyi', 'induced_p'],
                  list(np.linspace(0.1, 0.9, 9)) + \
                  [1., 1.5, 2., 2.5, 3., 3.5, 4., 4.5, 5., 5.5, 6., 7.],
                  'g50c',
                  allow_redraw=True,
                  test_data = test_data,
                  test_annot = test_annot,
                  resfpref = '_val_test')
