# -*- coding: utf-8 -*-
"""
@author: Christoph Lassner
"""
import os
os.chdir(os.path.dirname(__file__))
import csv
import numpy as np
np.random.seed(1)
import sklearn.grid_search as grid_search
from sklearn.cross_validation import ShuffleSplit, train_test_split
import fertilized_sklearn_helpers as helpers
from evaluation_helper import evaluate_crossval
#%% Load data.
with open('regression-datasets-housing.csv', 'r') as dataf:
  reader = csv.reader(dataf)
  rowvals = []
  for row in reader:
    rowvals.append([float(x) for x in row])
DTAARR = np.array(rowvals, dtype='float32')
train_data, test_data, train_annot, test_annot = train_test_split(DTAARR[:, :-1],
                                                                  DTAARR[:, -1:],
                                                                  test_size=51)
n_features = train_data.shape[1]
helpers._N_THREADS = 15
soil = helpers._REG_SOIL_
#%% Grid search.
#param_grid = {'max_depth':[5, 10, 15], # 15
#              'features_per_node':[4, 7, 10], # 4
#              'thresholds_per_node':[3, 5, 7]} # 3
#rand_init = 1
#origin = helpers.RegForestWrapper(
#                       n_features,
#                       25,  # max depth
#                       int(np.sqrt(n_features)), # n features per node
#                       10,  # n random thresholds per node
#                       100, # n trees
#                       4,   # min samples per leaf
#                       8,   # min samples per split
#                       0.,  # min gain threshold
#                       True, # allow redraw,
#                       rand_init,   # random seed
#                       'shannon', # entropy type
#                       2.    # induced entropy param
#                       )
#cv = ShuffleSplit(train_data.shape[0],
#                  n_iter=100,
#                  test_size=0.25,
#                  random_state=rand_init)
#clf = grid_search.GridSearchCV(origin,
#                               param_grid,
#                               'mean_squared_error',
#                               cv=cv,
#                               verbose=10,
#                               refit=True)
#clf.fit(train_data, train_annot)
#print 'Best parameters determined: ', clf.best_params_
#best_params = clf.best_params_
# Our result:
best_params = {'thresholds_per_node': 3,
               'features_per_node': 4,
               'max_depth': 15}
#%% Evaluation.
evaluate_crossval(soil,
                  train_data,
                  train_annot,
                  250, # val iter
                  51, # val size
                  best_params['max_depth'], #10, # max depth
                  best_params['features_per_node'], #8, #int(np.round(np.sqrt(n_features))), # 8 # features
                  best_params['thresholds_per_node'], #3, # thresholds per node
                  100, # n trees
                  8, # min samples split
                  4, # min samples leaf
                  ['shannon', 'renyi', 'tsallis', 'induced_p'],
                  list(np.linspace(0.1, 0.9, 9)) + \
                  [1.1, 1.5, 2., 2.5, 3., 3.5, 4., 4.5, 5., 6., 7., 8., 9.],
                  'boston_housing',
                  allow_redraw=True,
                  test_data = test_data,
                  test_annot= test_annot,
                  resfpref = '_val_test')
